'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.revertDiffs = exports.revertChanges = exports.applyDiffs = exports.applyChanges = undefined;

var _deepDiff = require('deep-diff');

/**
 * Calls `applyChange` or `revertChange` on the target object for
 * each change
 * @param {Function} applyOrRevertChange - `applyChange` or `revertChange`
 * @param {Object} target - target object to apply changes to
 * @param {[Object]} changes - array of `deep-diff` change objects
 * @returns {Object} the given target
 */
function doChanges(applyOrRevertChange, target) {
  var changes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  for (var i = 0, l = changes.length; i < l; i++) {
    // second argument to applyChange/revertChange is unused
    applyOrRevertChange(target, target, changes[i]);
  }

  return target;
}

/**
 * Calls `applyChange` or `revertChange` on the target object for
 * each change in each diff
 * @param {Function} applyOrRevertChange - `applyChange` or `revertChange`
 * @param {Object} target - target object to apply changes to
 * @param {[Array]} changes - array of diffs each containing
 * `deep-diff` change objects
 * @returns {Object} the given target
 */
function doDiffs(applyOrRevertChange, target) {
  var diffs = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  for (var i = 0, l = diffs.length; i < l; i++) {
    doChanges(applyOrRevertChange, target, diffs[i]);
  }

  return target;
}

// bind `applyChange` and `revertChange` to the functions above
var applyChanges = exports.applyChanges = doChanges.bind(null, _deepDiff.applyChange);
var applyDiffs = exports.applyDiffs = doDiffs.bind(null, _deepDiff.applyChange);
var revertChanges = exports.revertChanges = doChanges.bind(null, _deepDiff.revertChange);
var revertDiffs = exports.revertDiffs = doDiffs.bind(null, _deepDiff.revertChange);